from mayan.apps.appearance.classes import Icon

icon_statistics = Icon(driver_name='fontawesome', symbol='sort-numeric-up')
icon_execute = Icon(driver_name='fontawesome', symbol='cog')
icon_namespace_details = Icon(driver_name='fontawesome', symbol='sort-numeric-up')
icon_namespace_list = Icon(driver_name='fontawesome', symbol='sort-numeric-up')
icon_view = Icon(driver_name='fontawesome', symbol='eye')
