from .classes import PythonDependency

PythonDependency(
    module=__name__, name='node-semver', version_string='==0.8.0'
)
PythonDependency(
    module=__name__, name='packaging', version_string='==20.3'
)
