{% extends 'appearance/base_plain.html' %}

{% load i18n %}

{% load documents_tags %}

{% block title %}{{ title }}{% endblock title %}

{% block content_plain %}
    {% for page in pages %}
        <img
            alt="{% trans 'Document page image preview' %}"
            src="{% documents_get_api_image_url page width=width height=height %}" style="width: 100%;"
        />
    {% endfor %}
{% endblock %}
