from mayan.apps.appearance.classes import Icon

icon_transformations = Icon(driver_name='fontawesome', symbol='crop')

icon_transformation_delete = Icon(driver_name='fontawesome', symbol='times')
icon_transformation_edit = Icon(driver_name='fontawesome', symbol='pencil-alt')
icon_transformation_list = icon_transformations
icon_transformation_select = Icon(driver_name='fontawesome', symbol='plus')
