from mayan.apps.appearance.classes import Icon

icon_event_types_subscriptions_list = Icon(
    driver_name='fontawesome', symbol='list-ol'
)
icon_events_for_object = Icon(driver_name='fontawesome', symbol='list-ol')
icon_events_list = Icon(driver_name='fontawesome', symbol='list-ol')
icon_object_event_types_user_subcriptions_list = Icon(
    driver_name='fontawesome', symbol='rss'
)
icon_user_notifications_list = Icon(
    driver_name='fontawesome', symbol='bell'
)
